package org.sample.nio.command;

import org.sample.nio.QuoteManager;
import org.sample.nio.connection.handler.ClientConnectionHandler;
import org.sample.nio.connection.handler.ThirdPartyConnectionHandler;
import org.sample.nio.response.handler.FeedResponseHandler;
import org.sample.nio.response.handler.IResponseHandler;

/**
 * The feed Command must be in the format
 * 
 * feed|symbol[eoq]
 * 
 * The server will return the latest quote from
 * the cache, if there is a feed subscribe for this
 * symbol.  If not, the server will ask the quote at
 * the 3th party, and when the response is received,
 * it will be send to be client.
 * 
 * The connection with the client won't be close like
 * the quote command.
 * 
 * @author Sebastien Dionne
 *
 */
public class FeedCommand extends QuoteCommand {
	
	public FeedCommand(QuoteManager manager) {
		super(manager);
	}

	/**
	 * process the query
	 * @param query query to send to 3th party
	 * @param clientConnectionHandler client that call the command
	 * @return CommandRequest the request 
	 */
	public ICommandRequest process(String query, ClientConnectionHandler clientConnectionHandler){
		
		// extract the quote from the query
		int index = query.indexOf(QuoteManager.ARG_SEP);
		if(index<=0){
			//error
			return null;
		}
		
		f_quoteSubscription = query.substring(index+QuoteManager.ARG_SEP.length());
		
		f_responseHandler = new FeedResponseHandler();
		f_responseHandler.setClientConnectionHandler(clientConnectionHandler);
		f_responseHandler.setCommand(this);
		
		
		// add the quoteSubscription to the Cache for this client
		f_manager.addQuoteSubcriptionForClientConnectionHandler(f_quoteSubscription, clientConnectionHandler);
		
		// look if the quoteSubscription is not in the cache
		IResponseHandler handler = f_manager.getResponseHandlerFromQuoteSubcription(f_quoteSubscription);
		if(handler!=null){
			System.out.println("QUOTE FEED FOUND IN CACHE = " + f_quoteSubscription);
			
			handler.sendLastUpdateToClient(clientConnectionHandler);
			return handler.getCommandRequest();
		} 
		
		ThirdPartyConnectionHandler connection = f_manager.getThirdPartyConnectionHandler();
		
		if (connection == null) {
			return null;
		}

		// Create a new request to the 3th party
		try {
			f_manager.addQuoteSubcription(f_quoteSubscription, f_responseHandler);
			return connection.sendFeedRequest(f_quoteSubscription, f_responseHandler);
		} catch (Exception e) {
			// here we could send a message to the client.. but it's a demo
			System.out.println(e);
		}
		
		return null;
	}

}
