package org.sample.nio.connection.handler;

import java.io.IOException;
import java.net.SocketAddress;
import java.nio.channels.SelectionKey;
import java.nio.channels.Selector;
import java.nio.channels.ServerSocketChannel;
import java.nio.channels.SocketChannel;

import org.sample.nio.QuoteManager;

/**
 * Handles new client connections.
 * An Acceptor is bound on a ServerSocketChannel objects, which can produce new
 * SocketChannels for new clients using its <CODE>accept</CODE> method.
 */
public class ConnectionAcceptor {
    protected Selector f_selector;
    protected ServerSocketChannel f_serverSocketChannel;
    protected QuoteManager f_manager;

    /**
     * Creates a new ConnectionAcceptor
     * @param selector the Selector to which new SocketChannels will be registered
     * @param ssChannel the ServerSocketChannel which can accept new connections
     * @param pool the thread pool, which is needed by the new ConnectionReaders
     */
    public ConnectionAcceptor(Selector selector, ServerSocketChannel ssChannel, QuoteManager manager) {
        f_selector = selector;
        f_serverSocketChannel = ssChannel;
        f_manager = manager;
    }

    /**
     * Accepts a connection:
     * <UL>
     * <LI>Creates a SocketChannel for it
     * <LI>Creates a ConnectionReader for it
     * <LI>Registers the SocketChannel and the ConnectionReader to the Selector
     * </UL>
     * @throws IOException in case of an IOException during the acceptance of a new connection
     */
    public void accept() throws IOException {
        // Get a new channel for the connection request
        SocketChannel socketChannel = f_serverSocketChannel.accept();

        // If serverSocketChannel is non-blocking, sChannel may be null
        if (socketChannel != null) {
            socketChannel.configureBlocking(false);
            socketChannel.register(f_selector, SelectionKey.OP_READ, new ClientConnectionHandler(f_manager, socketChannel));
        }
    }
}

